@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/auth/cellconfig.c,v 1.14.2.7 2002/09/10 22:33:39 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/auth/cellconfig.c,v 1.14.2.10 2004/10/18 05:08:04 shadow Exp $");
 
 #include <afs/stds.h>
 #include <afs/pthread_glock.h>
@@ -35,6 +35,9 @@ RCSID("$Header: /cvs/openafs/src/auth/cellconfig.c,v 1.14.2.7 2002/09/10 22:33:3
 #include <sys/time.h>
 #ifdef AFS_AFSDB_ENV
 #include <arpa/nameser.h>
+#ifdef HAVE_ARPA_NAMESER_COMPAT_H
+#include <arpa/nameser_compat.h>
+#endif
 #include <resolv.h>
 #endif /* AFS_AFSDB_ENV */
 #endif /* AFS_NT40_ENV */
@@ -512,6 +515,7 @@ char clones[];
 	adir->alias_entries = curAlias;
     }
 
+    if (tf != NULL) fclose(tf);
     /* now read the fs keys, if possible */
     adir->keystr = (struct afsconf_keys *) 0;
     afsconf_IntGetKeys(adir);
@@ -785,7 +789,7 @@ int afsconf_GetAfsdbInfo(acellName, aservice, acellInfo)
     if (aservice) {
         LOCK_GLOBAL_MUTEX
         tservice = afsconf_FindService(aservice);
-     UNLOCK_GLOBAL_MUTEX
+	UNLOCK_GLOBAL_MUTEX
         if (tservice < 0) {
             return AFSCONF_NOTFOUND;  /* service not found */
      }
@@ -1023,8 +1027,10 @@ struct afsconf_keys *astr;
 
     LOCK_GLOBAL_MUTEX
     code = afsconf_Check(adir);
-    if (code)
+    if (code) {
+        UNLOCK_GLOBAL_MUTEX
 	return AFSCONF_FAILURE;
+    }
     memcpy(astr, adir->keystr, sizeof(struct afsconf_keys));
     UNLOCK_GLOBAL_MUTEX
     return 0;
@@ -1045,8 +1051,10 @@ afs_int32 afsconf_GetLatestKey(adir, avno, akey)
     
     LOCK_GLOBAL_MUTEX
     code = afsconf_Check(adir);
-    if (code)
+    if (code) {
+        UNLOCK_GLOBAL_MUTEX
 	return AFSCONF_FAILURE;
+    }
     maxa = adir->keystr->nkeys;
 
     best = -1;	    /* highest kvno we've seen yet */
@@ -1080,8 +1088,10 @@ char *akey;
 
     LOCK_GLOBAL_MUTEX
     code = afsconf_Check(adir);
-    if (code)
+    if (code) {
+        UNLOCK_GLOBAL_MUTEX
 	return AFSCONF_FAILURE;
+    }
     maxa = adir->keystr->nkeys;
 
     for(tk = adir->keystr->key,i=0;i<maxa;i++,tk++) {
