@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/rx/SOLARIS/rx_knet.c,v 1.6.2.5 2003/01/11 07:49:00 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/rx/SOLARIS/rx_knet.c,v 1.6.2.6 2003/04/14 15:47:03 shadow Exp $");
 
 #ifdef AFS_SUN5_ENV
 #include "../rx/rx_kcommon.h"
@@ -299,7 +299,7 @@ int osi_FreeSocket(asocket)
     AFS_STATCNT(osi_FreeSocket);
 
     taddr.sin_family = AF_INET;
-    taddr.sin_port = htons(rx_port);
+    taddr.sin_port = rx_port;
     taddr.sin_addr.s_addr = htonl(0x7f000001);
 
     dvec.iov_base = &c;
